% The codes collect data from files containing the features extracted
% from different signals and perform the k-folds validation with the DPGMM models
% can model features as multi-variable or single variable with expert vote
% **********************************************************************
dataSet = 'ECG_DB';
fileDir = fullfile('../Features4HealthySleep/Wavelet/AllFeatPro_novlp_ds/',dataSet);
if strcmp(dataSet,'St_Vincents')
    respLv = 3; % problems equal and worse will be considered: 1--possible, 3--hyp, 4--apnea
elseif strcmp(dataSet,'ECG_DB')
    respLv = 1;
end
% trainPropt = 0.7;

featAll = [];
gtAll = [];
files = dir(fileDir);
for fileInd = 1:length(files)
    if files(fileInd).isdir
        continue;
    end
    fileName = files(fileInd).name;
    load(fullfile(fileDir,fileName),'featStruct');
    featAll = cat(1,featAll,featStruct.feat);
    gtAll = cat(1,gtAll,featStruct.segGt);
end

% ind = ff2n(length(featStruct.featNames));
% % ind = ff2n(7);
% ind = ind(2:end,:);
% tmp = 1:length(featStruct.featNames);
% % tmp = 1:7;
% featSubset = arrayfun(@(i)tmp(logical(ind(i,:))),1:size(ind,1),'un',0)';
% clear tmp;clear ind;
featNum = length(featStruct.featNames);

% perfStruct = struct('feats',cell(2,1),'accuracy', [], ...
%                                     'sensitivity', [], 'specificity', []);

allFeats = featStruct.featNames;
selectFeats = {'t_M2','MinV','Power_Wavdb1D_2','Wav_db1_L3Num','Wav_db1_L2Num','Rg'};
% selectFeats = {'Power_Wavdb1D_2','Wav_db1_L3Num','Wav_db1_L2Num'};    
selectInd = arrayfun(@(j)find(arrayfun(@(i)strcmp(selectFeats{j}, allFeats{i}),...
    1:length(allFeats))),1:length(selectFeats));
prbFeat = featAll(bsxfun(@and,gtAll>=respLv, ~isnan(featAll(:,2))),selectInd); % attention for NaN check
normalFeat = featAll(bsxfun(@and,gtAll==0, ~isnan(featAll(:,2))),selectInd);
prbNum = size(prbFeat,1);
normalNum = size(normalFeat,1);
gtLabel = cat(1,true(prbNum,1),false(normalNum,1));

% cross validation with k-folds
seed = 100; s = rng(seed,'twister'); % make sure each feat are trained and the tested on the same partitions
cp = cvpartition(gtLabel,'k',10); % Stratified cross-validation
order = unique(gtLabel);
% 
f = @(xTrain, yTrain, xTest,yTest)confusionmat(yTest,...
    DP_Classify(xTrain, yTrain, xTest, 'multiVari'),'order',order);
% SVM for comparison
% f = @(xTrain, yTrain, xTest,yTest)confusionmat(yTest,...
%     SVM_Classify(xTrain, yTrain, xTest),'order',order);
% Discriminant Analysis
% f = @(xtr,ytr,xte,yte)confusionmat(yte,...
% classify(xte,xtr,ytr),'order',order);


cfMat = crossval(f,cat(1,prbFeat,normalFeat),gtLabel,'partition',cp);
cfMat = reshape(sum(cfMat),2,2);
% confusionchart(cfMat,order)
sens = cfMat(order==true,order==true)/sum(cfMat(order==true,:));
spec = cfMat(order==false,order==false)/sum(cfMat(order==false,:));
acc = trace(cfMat)/sum(cfMat,'All');
fprintf('10-folds Accuracy:%f, Sensitivity:%f, Specificity:%f\n',acc, sens, spec);
perfStruct.feats = selectFeats;
perfStruct.accuracy = acc;
perfStruct.sensitivity = sens;
perfStruct.specificity = spec;
 
save(strcat('Results_',dataSet,'_ovlp15_multiVari_6feat_best.mat'),'perfStruct');
