% The codes collect data from files containing the features extracted
% from different signals and perform the k-folds validation with the DPGMM models
% **********************************************************************
% dataSet = 'St_Vincents';
dataSet = 'St_Vincents';
fileDir = fullfile('../Features4HealthySleep/Wavelet/db1_L23_num/',dataSet);
if strcmp(dataSet,'St_Vincents')
    respLv = 3; % problems equal and worse will be considered: 1--possible, 3--hyp, 4--apnea
elseif strcmp(dataSet,'ECG_DB')
    respLv = 1;
end

wavTh = [0.4 0.5 0.55 0.6 0.7;
         0.5 0.6 0.65 0.7 0.8];
thNum = size(wavTh,2);
     
featAll = [];
gtAll = [];
files = dir(fileDir);
for fileInd = 1:length(files)
    if files(fileInd).isdir
        continue;
    end
    fileName = files(fileInd).name;
    load(fullfile(fileDir,fileName),'featStruct');
    featAll = cat(1,featAll,featStruct.feat);
    gtAll = cat(1,gtAll,featStruct.segGt);
end


perfStruct = struct('feats',cell(thNum,thNum),'thL2',[],'thL3',[],...
                    'accuracy', [], 'sensitivity', [], 'specificity', []);

for thInd_L2 = 1:thNum
    for thInd_L3 = 1:thNum
        prbFeat = cat(2,featAll(bsxfun(@and,gtAll>=respLv, ~isnan(featAll(:,1,thInd_L2))),1,thInd_L2),...
            featAll(bsxfun(@and,gtAll>=respLv, ~isnan(featAll(:,2,thInd_L3))),2,thInd_L3)); % attention for NaN check
        normalFeat = cat(2,featAll(bsxfun(@and,gtAll==0, ~isnan(featAll(:,1,thInd_L2))),1,thInd_L2),...
            featAll(bsxfun(@and,gtAll==0, ~isnan(featAll(:,2,thInd_L3))),2,thInd_L3));
        prbNum = size(prbFeat,1);
        normalNum = size(normalFeat,1);
        gtLabel = cat(1,true(prbNum,1),false(normalNum,1));

        % cross validation with k-folds
        seed = 100; s = rng(seed,'twister'); % make sure each feat are trained and the tested on the same partitions
        cp = cvpartition(gtLabel,'k',10); % Stratified cross-validation
        order = unique(gtLabel);

        f = @(xTrain, yTrain, xTest,yTest)confusionmat(yTest,DP_Classify(xTrain, yTrain, xTest),'order',order);

        cfMat = crossval(f,cat(1,prbFeat,normalFeat),gtLabel,'partition',cp);
        cfMat = reshape(sum(cfMat),2,2);
        % confusionchart(cfMat,order)
        sens = cfMat(order==true,order==true)/sum(cfMat(order==true,:));
        spec = cfMat(order==false,order==false)/sum(cfMat(order==false,:));
        acc = trace(cfMat)/sum(cfMat,'All');
        fprintf('10-folds Accuracy:%f, Sensitivity:%f, Specificity:%f\n',acc, sens, spec);
        perfStruct(thInd_L2,thInd_L3).feats = {featStruct.featNames{featSubset{subsetInd}}};
    %     perfStruct(subsetInd).feats = featSubset{subsetInd}; % for wavelet detail coef energy as feats
        perfStruct(thInd_L2,thInd_L3).thL2 = wavTh(1,thInd_L2);
        perfStruct(thInd_L2,thInd_L3).thL3 = wavTh(2,thInd_L3);
        perfStruct(thInd_L2,thInd_L3).accuracy = acc;
        perfStruct(thInd_L2,thInd_L3).sensitivity = sens;
        perfStruct(thInd_L2,thInd_L3).specificity = spec;
    end
end
save(fullfile('./Results',strcat('Results_',dataSet,'_wav_db1_L23_ROC_comb.mat')),'perfStruct');

% find the optimal ths
maxAcc = 0; maxSens = 0;
thL2_maxAcc = 0; thL2_maxSens = 0;
thL3_maxAcc = 0; thL3_maxSens = 0;
for thInd_L2 = 1:thNum
    for thInd_L3 = 1:thNum
        if  perfStruct(thInd_L2,thInd_L3).accuracy > maxAcc
            maxAcc = perfStruct(thInd_L2,thInd_L3).accuracy;
            thL2_maxAcc = perfStruct(thInd_L2,thInd_L3).thL2; 
            thL3_maxAcc = perfStruct(thInd_L2,thInd_L3).thL3;
        end
        if  perfStruct(thInd_L2,thInd_L3).sensitivity > maxSens
            maxSens = perfStruct(thInd_L2,thInd_L3).sensitivity;
            thL2_maxSens = perfStruct(thInd_L2,thInd_L3).thL2; 
            thL3_maxSens = perfStruct(thInd_L2,thInd_L3).thL3;
        end
    end
end