% The codes perform k-folds validation to choose a threshold for the wavelet 
% detaild coefficients
% **********************************************************************
% dataSet = 'St_Vincents';
dataSet = 'St_Vincents';
fileDir = fullfile('../Features4HealthySleep/Wavelet/L5Num_ovlp_15/',dataSet);
if strcmp(dataSet,'St_Vincents')
    respLv = 3; % problems equal and worse will be considered: 1--possible, 3--hyp, 4--apnea
elseif strcmp(dataSet,'ECG_DB')
    respLv = 1;
end

wavTh = 2.5:0.1:4;
thNum = size(wavTh,2);
     
featAll = [];
gtAll = [];
files = dir(fileDir);
for fileInd = 1:length(files)
    if files(fileInd).isdir
        continue;
    end
    fileName = files(fileInd).name;
    load(fullfile(fileDir,fileName),'featStruct');
    featAll = cat(1,featAll,featStruct.feat);
    gtAll = cat(1,gtAll,featStruct.segGt);
end


perfStruct = struct('feats',cell(thNum,1),'th',[],...
                    'accuracy', [], 'sensitivity', [], 'specificity', []);

for thInd= 1:thNum 
    prbFeat = featAll(bsxfun(@and,gtAll>=respLv, ~isnan(featAll(:,thInd))),thInd); % attention for NaN check
    normalFeat = featAll(bsxfun(@and,gtAll==0, ~isnan(featAll(:,thInd))),thInd);
    prbNum = size(prbFeat,1);
    normalNum = size(normalFeat,1);
    gtLabel = cat(1,true(prbNum,1),false(normalNum,1));

    % cross validation with k-folds
    seed = 100; s = rng(seed,'twister'); % make sure each feat are trained and the tested on the same partitions
    cp = cvpartition(gtLabel,'k',10); % Stratified cross-validation
    order = unique(gtLabel);

    f = @(xTrain, yTrain, xTest,yTest)confusionmat(yTest,DP_Classify(xTrain, yTrain, xTest,'multiVari'),'order',order);

    cfMat = crossval(f,cat(1,prbFeat,normalFeat),gtLabel,'partition',cp);
    cfMat = reshape(sum(cfMat),2,2);
    % confusionchart(cfMat,order)
    sens = cfMat(order==true,order==true)/sum(cfMat(order==true,:));
    spec = cfMat(order==false,order==false)/sum(cfMat(order==false,:));
    acc = trace(cfMat)/sum(cfMat,'All');
    fprintf('10-folds Accuracy:%f, Sensitivity:%f, Specificity:%f\n',acc, sens, spec);
    perfStruct(thInd).feats = {featStruct.featNames};
%     perfStruct(subsetInd).feats = featSubset{subsetInd}; % for wavelet detail coef energy as feats
    perfStruct(thInd).th = wavTh(thInd);
    perfStruct(thInd).accuracy = acc;
    perfStruct(thInd).sensitivity = sens;
    perfStruct(thInd).specificity = spec;
end
save(fullfile('./Results',strcat('Results_',dataSet,'_wav_db1_L5th.mat')),'perfStruct');